/*	te_pmd.c

	Text editor -- version for the PMD 85.

	Copyright (c) 2018 Michal Jurica / ZeroTeam

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2, or (at your option) any
	later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

	Usage:

	te [filename]

	Compilation:

	cc te_pmd.c
	as te_pmd.asm
	as crtcllin.asm
	ln -o te.com te_pmd.o crtcllin.o -lc

	Changes:

	09 Jan 2018 : 1st version.

	Notes:

	The PMD 85 runs on Mikros (CP/M v2) with 24x48 minus 1 line for
       	system messages (it can be enabled or disabled).
*/

/* Definitions
   -----------
*/
#define CRT_NAME "PMD 85 & Mikros"

#define CRT_ROWS 26        /* CRT rows: 24 - 1 (system line) */
#define CRT_COLS 48        /* CRT columns */

#define RULER_TAB     0x91 /* Ruler: Tab stop character - ie: ! */
#define RULER_CHR     0x90 /* Ruler: Character - ie: . */
#define SYS_LINE_SEP  0x9A /* System line separator character - ie: - */

/* Keys
   ----
*/

#define CRT_ESC_KEY "K0" /* ESCAPE key name */

#define K_UP	 0x1A /* Ctl _ */
#define K_DOWN	 0x19 /* Ctl ^ */
#define K_LEFT	 0x08  /* Ctl A */
#define K_RIGHT	 0x18  /* Ctl F */

#define K_PGUP	 0x17 /* Ctl Q */
#define K_PGDOWN 0x1B /* Ctl Z */

#define K_BEGIN	 0x0C /* Ctl V */
#define K_END	 0x03 /* Ctl \ */

#define K_TOP    145 /* Ctl P */
#define K_BOTTOM 146 /* Ctl S */

#define K_TAB    30  /* Ctl I */

#define K_INTRO	 13 /* Ctl M */
#define K_ESC	 136 /* Ctl [ */

#define K_RDEL	 29  /* Ctl G */
#define K_LDEL   147

#define K_CUT    137 /* Ctl U */
#define K_COPY   138 /* Crl R */
#define K_PASTE  139 /* Ctl W */

/* Help
   ----
*/

#define H_0 "Up     [UP]        Left   [LEFT]"
#define H_1 "Down   [DOWN]      Right  [RIGHT]"
#define H_2 "Begin  [\\]         LtDel  [K11]"
#define H_3 "End    [END]       RtDel  [DEL]"
#define H_4 "Top    [K9]        PgUp   [RCL]"
#define H_5 "Bottom [K10]       PgDown [CLR]"
#define H_6 ""
#define H_7 "Cut    [K1]"           
#define H_8 "Copy   [K2]        Intro  [ENTER]"
#define H_9 "Paste  [K3]        Esc    [K0]"

/* Include main code
   -----------------
*/
#include "te.c"
#include "sgtty.h"

/* Setup CRT: Used when the editor starts
   --------------------------------------
   void CrtSetup(void)
*/
CrtSetup()
{
	FILE *fp;
	struct sgttyb stty;

	if ((fp = fopen("con:", "r")) == NULL){
		printf("can't open the console\n");
		exit();
	}
	stty.sg_flags = RAW;
	ioctl(fileno(fp), TIOCSETP, &stty);
}

/* Reset CRT: Used when the editor exits
   -------------------------------------
   void CrtReset(void)
*/
CrtReset()
{
}

/* Output character to the CRT
   ---------------------------
   All program output is done with this function.

   On '\n' outputs '\n' + '\r'.

   void CrtOut(int ch)
#asm
CrtOut:
	mov a,l
	cpi 10
	jnz CrtOut2
	call CrtOut2
	mvi l,13
CrtOut2:
	mov c,l
	lhld 1
	lxi d,9
	dad d
	pchl	
#endasm
*/

/* Input character from the keyboard
   ---------------------------------
   All program input is done with this function.

   May be used to translate IBM PC key codes into single characters.

   int CrtIn(void)
{
#asm
CrtIn:
	lhld 1
	lxi d,6
	dad d
	lxi d,CrtIn2
	push d
	pchl
CrtIn2:
	mvi h,0
	mov l,a
	ret
#endasm
}
*/

/* Clear screen and send cursor to 0,0
   -----------------------------------
   void CrtClear(void)
*/
CrtClear()
{
	putchar(0x1F);
}

/* Locate the cursor (HOME is 0,0)
   -------------------------------
   void CrtLocate(int row, int col)
*/
CrtLocate(row, col)
int row, col;
{
	putchar(0x1B); putchar(row + 32); putchar(col + 32);
}

/* Erase line and cursor to row,0
   ------------------------------
   void CrtClLine(int row)
*/

/*
CrtClLine(row)
int row;
{
	char i;
	CrtLocate(row, 0);
	for (i = 0; i < 48; i++)
		putchar(' ');
	CrtLocate(row, 0);
}
*/


/* Erase from the cursor to the end of the line
   --------------------------------------------
*/
CrtClEol()
{
	/*putchar(27); putchar('K');*/
	CrtClEnd(BOX_ROW+box_shr, box_shc);
}

